@@ -2,49 +2,84 @@
 
 import atexit
 import logging
-import redis, threading, traceback, sys
+import redis, threading
+import signal
+import traceback, sys, os
 
-from configator.engine import RedisClient
+from configator.engine.connector import RedisClient
+from configator.utils.function import assure_not_null
 from typing import Any, Callable, List, Tuple, Dict, Optional
 
 LOG = logging.getLogger(__name__)
 
 class SettingSubscriber(RedisClient):
     #
-    def __init__(self, *args, auto_stop=True, **kwargs):
-        if auto_stop:
-            atexit.register(self.stop)
+    def __init__(self, *args, **kwargs):
         super(SettingSubscriber, self).__init__(**kwargs)
         self.CHANNEL_PATTERN = self.CHANNEL_GROUP + '*'
     #
     ##
     @property
     def pubsub(self):
-        ps = self.connect().pubsub()
+        ps = assure_not_null(self.connect()).pubsub()
         ps.psubscribe(**{self.CHANNEL_PATTERN: self.__process_event})
         return ps
     #
     ##
-    __t = None
+    __pubsub_thread = None
+    __pubsub_lock = threading.RLock()
     #
     def start(self):
-        if self.__t is None:
-            self.__t = self.__run_in_thread(sleep_time=0.001)
-            if LOG.isEnabledFor(logging.DEBUG):
-                LOG.log(logging.DEBUG, "SettingSubscriber has started")
-        return self.__t
+        with self.__pubsub_lock:
+            if self.__pubsub_thread is None:
+                self.__pubsub_thread = self.__run_in_thread(sleep_time=0.001)
+                if LOG.isEnabledFor(logging.DEBUG):
+                    LOG.log(logging.DEBUG, "SettingSubscriber has started")
+            return self.__pubsub_thread
     #
     def stop(self):
-        if self.__t is not None:
-            self.__t.stop()
-            self.__t = None
-        self._destroy()
-        if LOG.isEnabledFor(logging.DEBUG):
-            LOG.log(logging.DEBUG, "SettingSubscriber has stopped")
+        return self.close()
     #
-    def __run_in_thread(self, sleep_time=0, daemon=False):
+    def close(self):
+        with self.__pubsub_lock:
+            if self.__pubsub_thread is not None:
+                self.__pubsub_thread.stop()
+            super().close()
+            if self.__pubsub_thread is not None:
+                self.__pubsub_thread.join()
+                self.__pubsub_thread = None
+            if LOG.isEnabledFor(logging.DEBUG):
+                LOG.log(logging.DEBUG, "SettingSubscriber has stopped")
+    #
+    #
+    def hook_atexit(self):
+        def atexit_handler():
+            if LOG.isEnabledFor(logging.DEBUG):
+                LOG.log(logging.DEBUG, "AtExit occurred")
+            self.close()
+        atexit.register(atexit_handler)
+        return self
+    #
+    def hook_sigint(self, finished=True):
+        return self.hook_signal(signal_code=signal.SIGINT, finished=finished)
+    #
+    def hook_signal(self, signal_code=signal.SIGINT, finished=False):
+        current_handler = None
+        def signal_handler(signalnum, frame):
+            if LOG.isEnabledFor(logging.DEBUG):
+                LOG.log(logging.DEBUG, "SIGNAL[%d] received" % signalnum)
+            self.close()
+            if not finished and callable(current_handler):
+                if LOG.isEnabledFor(logging.DEBUG):
+                    LOG.log(logging.DEBUG, "Invoke the default handler")
+                current_handler(signalnum, frame)
+        current_handler = signal.signal(signal_code, signal_handler)
+        return self
+    #
+    def __run_in_thread(self, auto_start=True, sleep_time=0, daemon=False):
         thread = PubSubWorkerThread(self, sleep_time, daemon=daemon)
-        thread.start()
+        if auto_start:
+            thread.start()
         return thread
     #
     ##
@@ -110,7 +145,9 @@ class PubSubWorkerThread(threading.Thread):
             except redis.ConnectionError:
                 self._redis_client.reconnect()
             except Exception as err:
-                traceback.print_exc(file=sys.stdout)
+                if LOG.isEnabledFor(logging.ERROR):
+                    LOG.log(logging.ERROR, err)
+                # traceback.print_exc(file=sys.stdout)
         if LOG.isEnabledFor(logging.DEBUG):
             LOG.log(logging.DEBUG, "PubSubWorkerThread has stopped")
     #
